/**
 * Copyright (c) 2011, REMPL.com
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1) Redistributions of source code must retain the above copyright
 * notice, this list of conditions and the following disclaimer.
 * 2) Redistributions in binary form must reproduce the above
 * copyright notice, this list of conditions and the following
 * disclaimer in the documentation and/or other materials provided
 * with the distribution.
 * 3) Neither the name of the REMPL.com nor the names of its
 * contributors may be used to endorse or promote products derived
 * from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT
 * NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 * FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL
 * THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT,
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 */
package com.rempl.api;

// java input/output
import java.io.File;
import java.io.IOException;

/**
 * Builder of a {@link Model}.
 *
 * @author Yegor Bugayenko (yegor@rempl.com)
 * @version $Id: Rempler.java 776 2011-03-18 22:35:07Z yegor256@yahoo.com $
 */
public interface Rempler {

    /**
     * Append new {@link Reader} to the list.
     * @param reader The {@link Reader} to use for processing
     */
    void append(final Reader reader);

    /**
     * Append new {@link Reporter} to the map.
     * @param reporter The {@link Reporter} to use
     * @param dir Relative location of the report
     */
    void append(final Reporter reporter, final String dir);

    /**
     * Builds a {@link Model}.
     * @return The {@link Model} just built
     * @throws java.io.IOException If IO problem occurs
     */
    Model read() throws IOException;

    /**
     * Makes an empty {@link Model}.
     * @return The {@link Model}
     */
    Model emptyModel();

    /**
     * Builds a set of reports for a {@link Model}.
     * @param model The {@link Model}
     * @param dir The directory where reports will be saved
     * @throws java.io.IOException If some IO problem occurs
     */
    void report(final Model model, final File dir) throws IOException;

    /**
     * Set entire configuration.
     * @param config The configuration
     */
    void configure(final ROMConfiguration config);

    /**
     * Read the entire configuration object.
     * @return The configuration
     */
    ROMConfiguration configuration();

}
