/**
 * @version $Id: ElementTest.java 1140 2011-04-04 16:37:58Z yegor256@yahoo.com $
 */
package com.rempl.api;

import org.junit.*;
import static org.junit.Assert.*;
import com.rempl.tk.*;
import com.rempl.impl.ElementImpl;
import java.util.Map;

public class ElementTest {

    @Test
    public void testSurfaceArea() throws Exception {
        Reflector.start()
            .interfaceExists("com.rempl.api.Element")
                .methodExists("has(String)")
                    .back() // to class
                .methodExists("get(String)")
                    .back() // to class
                .methodExists("all(String)")
                    .back() // to class
                .methodExists("put(String, String)")
                    .back() // to class
                    ;
    }

    @Test
    public void testSimpleOperations() throws Exception {
        Element e = new ElementImpl("c1", "Class");
        assertEquals("c1", e.name());
        assertEquals("Class", e.type());
        assertFalse(e.has("name"));
        e.put("name", "Employee");
        assertTrue(e.has("name"));
        assertEquals("Employee", e.get("name"));
        e.put("parent", "Person");
        e.put("parent", "Worker");
        assertTrue(e.has("parent"));
        assertTrue(e.all("parent").size() == 2);
        for (Map.Entry<String, String> entry : e.entrySet()) {
            assertTrue(e.has(entry.getKey()));
        }
    }

    @Test(expected = IllegalArgumentException.class)
    public void testAttemptsToGetNonExistingSlot() throws Exception {
        Element e = new ElementImpl("c1", "Class");
        e.get("parent");
    }

}
