/**
 * @version $Id: ModelTest.java 1614 2011-07-10 19:05:43Z guard $
 */
package com.rempl.api;

import com.rempl.impl.ModelImpl;
import com.rempl.tk.Reflector;
import com.ymock.util.Logger;
import org.junit.Test;

import java.io.StringReader;
import java.io.StringWriter;
import java.util.Collection;

import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertTrue;

public class ModelTest {

    @Test
    public void testSurfaceArea() throws Exception {
        Reflector.start()
            .interfaceExists("com.rempl.api.Model")
                .methodExists("transform(String)")
                    .back() // to interface
                .methodExists("query(String)")
                    .back() // to interface
                .methodExists("load(java.io.Reader)")
                    .back() // to interface
                .methodExists("save(java.io.Writer)")
                    .back() // to interface
                    ;
    }

    @Test
    public void testModelTransformQuery() throws Exception {
        Model model = new ModelImpl();
        model.transform(":Class { name: \"java.lang.String\"; };");
        Collection<Element> elements = model.query("context Class inv: true");
        for (Element e : elements) {
            assertTrue(e.has("name"));
            assertEquals("java.lang.String", e.get("name"));
        }
    }

    @Test
    public void testSavesAndLoadsXMI() throws Exception {
        Model modelSaved = new ModelImpl();
        modelSaved.transform(":Class { name: \"java.lang.String\"; };");
        modelSaved.transform(":Class { name: \"java.lang.Integer\"; };");
        StringWriter writer = new StringWriter();
        modelSaved.save(writer);
        Model modelLoaded = new ModelImpl();
        String xmi = writer.toString();
        Logger.debug(this, "XMI:\n%s", xmi);
        modelLoaded.load(new StringReader(xmi));
        Collection<Element> elements = modelLoaded.query("context Class inv: true");
        assertEquals("Nothing loaded back, why?", 2, elements.size());
        for (Element e : elements) {
            assertTrue(e.has("name"));
        }
    }

}
