/**
 * @version $Id: RemplTest.java 1614 2011-07-10 19:05:43Z guard $
 */
package com.rempl.api;

import com.ymock.util.RuntimeProblem;
import org.junit.*;
import org.junit.rules.*;
import static org.junit.Assert.*;
import com.rempl.tk.*;
import java.net.URL;
import java.io.File;
import org.apache.commons.io.FileUtils;

public class RemplTest {

    @Rule
    public TemporaryFolder tempFolder = new TemporaryFolder();

    @Test
    public void testSurfaceArea() throws Exception {
        Reflector.start()
            .interfaceExists("com.rempl.api.Rempl")
                .methodExists("getFactory()")
                    .returns("com.rempl.api.RemplFactory")
                    .accepts("void")
                    .isStatic()
                    .back() // to class
                    ;
    }

    @Test
    public void testValidatesStaticFactoryMethod() throws Exception {
        RemplFactory factory = Rempl.getFactory();
        assertNotNull(factory);
        assertTrue(factory instanceof com.rempl.impl.RemplFactoryImpl);
    }

    private static interface Mocker {
        void mockFile(final File file) throws Exception;
    }

    @Test
    public void testWhatHappensIfRemplFactoryIsNotConfigured() throws Exception {
        this.getFactoryThroughMocker(
            new Mocker() {
                public void mockFile(final File file) throws Exception {
                    file.delete();
                }
            }
        );
    }

    @Test
    public void testWhatHappensIfRemplFactoryIsBadlyConfigured() throws Exception {
        this.getFactoryThroughMocker(
            new Mocker() {
                public void mockFile(final File file) throws Exception {
                    FileUtils.writeStringToFile(file, "invalid.class.name");
                }
            }
        );
    }

    @Test
    public void testWhatHappensIfRemplFactoryIsConfiguredWithWrongClass() throws Exception {
        this.getFactoryThroughMocker(
            new Mocker() {
                public void mockFile(final File file) throws Exception {
                    FileUtils.writeStringToFile(file, "java.lang.String");
                }
            }
        );
    }

    private void getFactoryThroughMocker(Mocker mocker) throws Exception {
        Rempl.clean();
        URL svc = Rempl.class.getResource("/META-INF/services/com.rempl.api.RemplFactory");
        File file = FileUtils.toFile(svc);
        File temp = this.tempFolder.newFile("temp");
        FileUtils.copyFile(file, temp);
        mocker.mockFile(file);
        String failure = null;
        try {
            RemplFactory factory = Rempl.getFactory();
            failure = "Exception was not thrown, why?";
        } catch (Exception ex) {
            if (!(ex instanceof RuntimeProblem)) {
                failure = ex.getMessage();
            }
        }
        FileUtils.copyFile(temp, file);
        if (failure != null) {
            fail(failure);
        }
    }

}
