/**
 * @version $Id: ElementImpl.java 1140 2011-04-04 16:37:58Z yegor256@yahoo.com $
 */
package com.rempl.impl;

import com.rempl.api.Element;
import com.rempl.api.Model;
import java.util.*;

public final class ElementImpl implements Element {

    private Map<String, List<String>> slots =
        new HashMap<String, List<String>>();

    private String name;

    private String type;

    public ElementImpl(final String pname, final String ptype) {
        this.name = pname;
        this.type = ptype;
    }

    @Override
    public String name() {
        return this.name;
    }

    @Override
    public String type() {
        return this.type;
    }

    @Override
    public boolean has(final String key) {
        return this.slots.containsKey(key);
    }

    @Override
    public boolean refsTo(final String key, final Element another) {
        for (String value : this.slots.get(key)) {
            if (value.equals(Model.REF_PREFIX + another.name())) {
                return true;
            }
        }
        return false;
    }

    @Override
    public String get(final String key) {
        if (!this.has(key)) {
            throw new IllegalArgumentException(
                "Invalid key in "
                + this.name
                + " (" + this.type + ")"
            );
        }
        return this.slots.get(key).get(0);
    }

    @Override
    public <T> T get(final String key, final Class<T> type) {
        return (T) this.get(key);
    }

    @Override
    public void put(final String key, final String value) {
        if (!this.has(key)) {
            this.slots.put(key, new ArrayList<String>());
        }
        this.slots.get(key).add(value);
    }

    @Override
    public Collection<String> all(final String key) {
        return this.slots.get(key);
    }

    @Override
    public Collection<Map.Entry<String, String>> entrySet() {
        final Collection<Map.Entry<String, String>> list =
            new ArrayList<Map.Entry<String, String>>();
        for (Map.Entry<String, List<String>> entry : this.slots.entrySet()) {
            for (String value : entry.getValue()) {
                list.add(
                    new HashMap.SimpleEntry<String, String>(
                        entry.getKey(),
                        value
                    )
                );
            }
        }
        return list;
    }

}
