/**
 * @version $Id: RemplerImpl.java 1614 2011-07-10 19:05:43Z guard $
 */
package com.rempl.impl;

import com.rempl.api.*;
import com.ymock.util.Logger;

import java.io.File;
import java.io.IOException;
import java.util.*;

public class RemplerImpl implements Rempler {

    private ROMConfiguration configuration;

    private List<Reader> readers = new ArrayList<Reader>();

    private Map<String, Reporter> reporters = new HashMap<String, Reporter>();

    @Override
    public void append(final Reader reader) {
        Logger.trace(this, "#append(%s)", reader.getClass().getName());
        this.readers.add(reader);
        reader.setRempler(this);
    }

    @Override
    public void append(final Reporter reporter, final String dir) {
        Logger.trace(this, "#append(%s, %s)", reporter.getClass().getName(), dir);
        this.reporters.put(dir, reporter);
        reporter.setRempler(this);
    }

    @Override
    public Model read() throws IOException {
        Logger.trace(this, "#read()");
        Model model = this.emptyModel();
        List<Integer> running = new ArrayList<Integer>();
        for (Integer i = 0; i < this.readers.size(); i++) {
            running.add(i);
        }
        Logger.debug(this, "%d readers to be executed", running.size());
        do {
            List<Integer> next = new ArrayList<Integer>();
            for (Integer i : running) {
                Reader reader = this.readers.get(i);
                String name = reader.getClass().getName();
                Logger.debug(this, "reading with %s (#%d)", name, i);
                if (reader.read(model)) {
                    Logger.debug(this, "%s (%d) needs more", name, i);
                    next.add(i);
                } else {
                    Logger.debug(this, "removing %s (%d) from list", name, i);
                }
            }
            running = next;
        } while (running.size() > 0);
        return model;
    }

    @Override
    public Model emptyModel() {
        return new ModelImpl();
    }

    @Override
    public void report(final Model model, final File dir) throws IOException {
        Logger.trace(this, "#report()");
        // todo
    }

    @Override
    public void configure(final ROMConfiguration config) {
        this.configuration = config;
    }

    @Override
    public ROMConfiguration configuration() {
        return this.configuration;
    }

}
